//============================================================================
//
//      AppFuncs.cpp - Enable sample editor application functions file
//
//      Copyright 1993-2003 Cypress Software Inc.
//      All Rights Reserved.
//
//		Use AppFuncs.bas to test this file
//
//============================================================================

/* Type Mappings	Enable		32 bit C/C++

					Integer		short
					Long		long
					Single		float
					Double		double
					String		BSTR
					Array		SAFEARRAY
					UDT			struct
*/

#include "stdafx.h"

#include "AppFuncs.h"

typedef struct /*tagComplex*/ {
	float r;
	float i;
} Complex;

//============================================================================
char *pDeclarations =	// Enable basic declarations - must match functions below!

"Type Complex\n"
" r As Single\n"
" i As Single\n"
"End Type\n"

"Declare Function Factorial (ByVal i As Integer) As Double\n"
"Declare Sub Alert(ByVal str$)\n"
"Declare Function UpperCaseByRef (Str As String) As String\n"
"Declare Sub RaiseError (ByVal nRTError%)\n"
"Declare Function GetSafeArray ( arr() ) As Long\n"
"Declare Function GetVariantSafeArray ( var ) As Long\n"
"Declare Function PassUDT (pComplex As Complex) As Long\n"
"Declare Function MyAbs App (ByVal Val As Double) As Double\n"
"Declare Sub MyReDim (ByVal Preserve As Boolean, ArrVar(), ByVal LBound As Long, ByVal UBound As Long)\n"
"Declare Sub LoadTable (ArrVar() As Double, Data)\n"

"Dim MyVar As Variant\n"
;

//============================================================================
inline RuntimeError( HANDLE hCtx, int iErrNum )
//============================================================================
{
	// Helper function for reporting Enable runtime errors
	return enaSetErrorCode( hCtx, ENS_RUNTIMEERROR, iErrNum, NULL );
}

//============================================================================
double CALLBACK Factorial( HANDLE hCtx, short i )
//============================================================================
{
	// Check the range of the input parameter
	if ( i < 0 )
		return RuntimeError( hCtx, 5 );	// Invalid procedure call or argument

	if ( i > 170 )
		return RuntimeError( hCtx, 6 );	// Overflow

	// Compute the factorial
	double d = 1;
	while ( i )
		d = d * i--;

	return d;
}

//============================================================================
DWORD CALLBACK WaitForProcessInputIdle( HANDLE hCtx, DWORD dwProcessId/*, DWORD dwMilliseconds*/ )
//============================================================================
{
	// Call this function with the process ID returned from the Shell function in
	// order to make sure that the Shell'd process is ready to accept input

	HANDLE hProcess = OpenProcess(
		PROCESS_ALL_ACCESS,		// access flag
		FALSE,					// handle inheritance option
		dwProcessId				// process identifier
	);

	DWORD rc = WaitForInputIdle(
		hProcess,				// handle to process
		INFINITE				// time-out interval
	);

	CloseHandle( hProcess );

	return rc;
}

//============================================================================
void Alert( HANDLE ctx, LPCSTR str )
//============================================================================
{
    MessageBox( 0, str, "Alert", MB_OK );
}

//============================================================================
BSTR CALLBACK UpperCaseByRef( HANDLE hCtx, BSTR *pbstrOriginal )
//============================================================================
{
    int cbOriginalLen = SysStringByteLen( *pbstrOriginal );
    BSTR bstrUpperCase = SysAllocStringLen( NULL, cbOriginalLen );
    LPSTR strSrcByRef = (LPSTR)*pbstrOriginal;
    LPSTR strDst = (LPSTR)bstrUpperCase;

    for( int i=0; i<=cbOriginalLen; i++ )
        *strDst++ = toupper( *strSrcByRef++ );

    SysReAllocString( pbstrOriginal, (BSTR)"Good Bye" );

    return bstrUpperCase;
}

//============================================================================
void CALLBACK RaiseError( HANDLE hCtx, short nRTError )
//============================================================================
{
	RuntimeError( hCtx, nRTError );
}

//============================================================================
long CALLBACK GetSafeArray( HANDLE hCtx, SAFEARRAY **ppSafeArray )
//============================================================================
{
//	The SAFEARRAYBOUND structure represents the bounds of one dimension of the array.
//	The lower bound of the dimension is represented by lLbound, and cElements 
//	represents the number of elements in the dimension.
	SAFEARRAYBOUND sab;
	sab.cElements = 10;
	sab.lLbound = 0;
	SAFEARRAY *&psa = *ppSafeArray;
	if ( psa == NULL )
		psa = SafeArrayCreate( VT_VARIANT, 1, &sab );
	else
		SafeArrayRedim( psa, &sab );

	// Initialize the values
	VARIANT *pv = NULL;
	HRESULT hresult = SafeArrayAccessData( *ppSafeArray, (void**)&pv );

	VariantClear( &pv[0] );
	pv[0].vt = VT_BSTR;
	pv[0].bstrVal = SysAllocString( L"String data: Zero" );

	VariantClear( &pv[1] );
	pv[1].vt = VT_I4;
	pv[1].lVal = 111111;

	VariantClear( &pv[2] );
	pv[2].vt = VT_BSTR;
	pv[2].bstrVal = SysAllocString( L"String data: Two" );

	VariantClear( &pv[3] );
	pv[3].vt = VT_I2;
	pv[3].iVal = 3333;

	SafeArrayUnaccessData( *ppSafeArray );

    return 0;       // Success
}

//============================================================================
long CALLBACK GetSafeArrayOfString( HANDLE hCtx, SAFEARRAY **ppSafeArray )
//============================================================================
{
//	The SAFEARRAYBOUND structure represents the bounds of one dimension of the array.
//	The lower bound of the dimension is represented by lLbound, and cElements 
//	represents the number of elements in the dimension.
	SAFEARRAYBOUND sab;
	sab.cElements = 10;
	sab.lLbound = 0;
	SAFEARRAY *&psa = *ppSafeArray;
	if ( psa == NULL )
		psa = SafeArrayCreate( VT_BSTR, 1, &sab );
	else
		SafeArrayRedim( psa, &sab );

	// Initialize the values
	BSTR *ps = NULL;
	HRESULT hresult = SafeArrayAccessData( *ppSafeArray, (void**)&ps );

	ps[0] = SysAllocString( L"String data: Zero" );
	ps[1] = SysAllocString( L"String data: One" );
	ps[2] = SysAllocString( L"String data: Two" );
	ps[3] = SysAllocString( L"String data: Three" );

	SafeArrayUnaccessData( *ppSafeArray );

    return 0;       // Success
}

//============================================================================
long CALLBACK GetVariantSafeArray( HANDLE hCtx, VARIANT *pVariant )
//============================================================================
{
	VariantClear( pVariant );
	pVariant->parray = NULL;

//	pVariant->vt = VT_VARIANT | VT_ARRAY;
//	return GetSafeArray( hCtx, &(pVariant->parray) );		// Allocates an array of variants

	pVariant->vt = VT_BSTR | VT_ARRAY;
	return GetSafeArrayOfString( hCtx, &(pVariant->parray) );// Allocates an array of bstrs
}

//============================================================================
long CALLBACK PassUDT( HANDLE hCtx, Complex *pComplex )
//============================================================================
{
	return 42;
}

//============================================================================
double CALLBACK MyAbs( HANDLE hCtx, double d )
//============================================================================
{
	return d < 0.0 ? -d : d;
}

#if 0	// Variant parameter version
//=============================================================================================
void CALLBACK MyReDim( HANDLE hCtx, BOOL fPreserve, VARIANT *pVariant, long lLBound, long lUBound )
//=============================================================================================
{

//	The SAFEARRAYBOUND structure represents the bounds of one dimension of the array.
//	The lower bound of the dimension is represented by lLbound, and cElements 
//	represents the number of elements in the dimension.

	SAFEARRAYBOUND sab;								// Setup the array bounds
	sab.lLbound = lLBound;
	sab.cElements = lUBound - lLBound + 1;			// Calculate the number of elements
	SAFEARRAY *&psa = pVariant->parray;

	if ( psa == NULL ) {
		VARTYPE vt = pVariant->vt & VT_TYPEMASK;	// Get the scalar type
		if ( vt > VT_UNKNOWN )
			RuntimeError( hCtx, 13 );				// Type mismatch
		else
			psa = SafeArrayCreate( vt, 1, &sab );	// Create the array for the first time
	}
	else {
		HRESULT hr = SafeArrayRedim( psa, &sab );	// ReDim the array
		if ( FAILED(hr) )
			enaSetErrorCode( hCtx, hr, 0, NULL );
		else if ( !fPreserve ) {
			SafeArrayDestroyData( psa );			// Purge the data
			SafeArrayAllocData( psa );
		}
	}
}
#else	// SAFEARRAY parameter version 
//=============================================================================================
void CALLBACK MyReDim( HANDLE hCtx, BOOL fPreserve, SAFEARRAY **ppSafeArray, long lLBound, long lUBound )
//=============================================================================================
{

//	The SAFEARRAYBOUND structure represents the bounds of one dimension of the array.
//	The lower bound of the dimension is represented by lLbound, and cElements 
//	represents the number of elements in the dimension.

	SAFEARRAYBOUND sab;								// Setup the array bounds
	sab.lLbound = lLBound;
	sab.cElements = lUBound - lLBound + 1;			// Calculate the number of elements
	SAFEARRAY *&psa = *ppSafeArray;

	if ( psa == NULL ) {
		psa = SafeArrayCreate( VT_BSTR, 1, &sab );	// Create the array for the first time
	}
	else {
		HRESULT hr = SafeArrayRedim( psa, &sab );	// ReDim the array
		if ( FAILED(hr) )
			enaSetErrorCode( hCtx, hr, 0, NULL );
		else if ( !fPreserve ) {
			SafeArrayDestroyData( psa );			// Purge the data
			SafeArrayAllocData( psa );
		}
	}
}
#endif

//=============================================================================================
void CALLBACK LoadTable( HANDLE hCtx, SAFEARRAY **ppSafeArray,  VARIANT *pVariant )
//=============================================================================================
{
	// This version of LoadTable supports only arrays Dim'd As Double

	SAFEARRAY *psa = *ppSafeArray;
	VARTYPE vt = VT_R8;								// Default to "As Double" for now
	HRESULT hr = SafeArrayGetVartype( psa, &vt );	// Only works with Enable 4.3

	// To do: Handle dynamic (not fixed size) arrays and types other than Double

	if ( FAILED(hr) && psa && psa->cbElements != sizeof(double) ) {
		enaSetErrorCode( hCtx, hr, 0, NULL );
		return;
	}

	// To do: Check if variant arg can be converted to a string

	if ( vt != VT_R8 || pVariant->vt != VT_BSTR ) {	// Current limitations
		RuntimeError( hCtx, 13 );					// Type mismatch
		return;
	}

	unsigned long dwFlags =	  NUMPRS_LEADING_PLUS	// Accept leading "+"
							| NUMPRS_LEADING_MINUS	// Accept leading "-"
							| NUMPRS_HEX_OCT		// Accept hex and octal (&h13)
							| NUMPRS_DECIMAL		// Accept decimal numbers (3.1415)
							| NUMPRS_EXPONENT		// Accept scientific notation (1.3e-5)
							;

	PWSTR pStr = pVariant->bstrVal;					// Pointer to the BSTR data in the VARIANT
	int nNestingLevel = 0;							// Check for balanced braces
	double *pDbl = NULL;

	// Find the total number of elements in the array
	int nElements = 1;
	for ( int cDims = psa->cDims; cDims-- > 0; )
		nElements *= psa->rgsabound[cDims].cElements;

	hr = SafeArrayAccessData( *ppSafeArray, (void**)&pDbl );
	if ( FAILED(hr) ) {
		enaSetErrorCode( hCtx, hr, 0, NULL );
		return;
	}

	double *pEnd = pDbl + nElements;				// Used to avoid buffer overrun

	while ( *pStr && SUCCEEDED(hr) ) {

		// To do: Improve handling of "{", "," and "}"

		switch ( *pStr ) {
		case '{':
			nNestingLevel++;						// Increment the nesting level
			break;

		case ' ':									// Ignore spaces
		case '\t':									// Ignore tabs
		case ',':									// Ignore commas
			break;

		case '}':
			--nNestingLevel;						// Decrement the nesting level
			break;

		default:
			if ( pDbl < pEnd ) {
				BYTE digits[30];					// Holds max digits of a double
				NUMPARSE numprs = {sizeof(digits),dwFlags};
				VARIANT v = {0};					// Temporary variant holds parsed double

				// Parse the number
				hr = VarParseNumFromStr( pStr, LOCALE_NEUTRAL, dwFlags, &numprs, digits );

				// Convert the parse structure to a variant
				if ( SUCCEEDED(hr) ) {
					ULONG dwVtBits = 1 << vt;		// Determine the type to convert to (double)
					hr = VarNumFromParseNum( &numprs, digits, dwVtBits, &v );
				}

				// Store the double in the array
				if ( SUCCEEDED(hr) ) {
					pStr += numprs.cchUsed;			// Advance past number token
					*pDbl++ = v.dblVal;				// Store double in the array
					continue;						// Back to loop top to continue parsing
				}
			}
			else
				hr = E_INVALIDARG;

			enaSetErrorCode( hCtx, hr, 0, NULL );
			continue;								// Need to unlock array
		}

		pStr++;
	}

	SafeArrayUnaccessData( *ppSafeArray );			// Decrement the lock count

	if ( SUCCEEDED(hr) && nNestingLevel )			// Braces not balanced
		RuntimeError( hCtx, 5 );					// Invalid procedure call or argument
}

//============================================================================
long CALLBACK GetAppProcAddr( HANDLE hCtx, LPCSTR pFuncName )	// Name to address mapping function
//============================================================================
{
    if		( !stricmp("Factorial", pFuncName) )			return (long)Factorial;
    else if ( !stricmp("UpperCaseByRef", pFuncName) )		return (long)UpperCaseByRef;
    else if ( !stricmp("Alert", pFuncName) )				return (long)Alert;
    else if ( !stricmp("RaiseError", pFuncName) )			return (long)RaiseError;
    else if ( !stricmp("GetSafeArray", pFuncName) )			return (long)GetSafeArray;
    else if ( !stricmp("GetVariantSafeArray", pFuncName) )	return (long)GetVariantSafeArray;
    else if ( !stricmp("PassUDT", pFuncName) )				return (long)PassUDT;
    else if ( !stricmp("MyAbs", pFuncName) )				return (long)MyAbs;
    else if ( !stricmp("MyReDim", pFuncName) )				return (long)MyReDim;
    else if ( !stricmp("LoadTable", pFuncName) )			return (long)LoadTable;

    return (long)NULL;	// Unknown function name
}
