//===========================================================================
//
//      EditFind.cpp - Script Editor Search and Replace Functions
//
//      Copyright 1993-2003 Cypress Software Inc.  All Rights Reserved.
//      Information Contained Herein Is Proprietary and Confidential.
//
//===========================================================================

#include "stdafx.h"

#include <richedit.h>			// Rich edit control prototypes and definitions


//===========================================================================
static UINT APIENTRY FindReplaceHook( HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam )
//===========================================================================
{
	static HWND hwndEdit;

	switch( msg ) {
	case WM_INITDIALOG:
		hwndEdit = ((FINDREPLACE *)lParam)->hwndOwner;
		return TRUE;
	}
	return FALSE;	// default processing
}

//===========================================================================
#define MAX_STRING_LEN   256

static char szFindText[MAX_STRING_LEN];
static char szReplText[MAX_STRING_LEN];

static FINDREPLACE fr = {       // must be static for modeless dialog!!!
   /*lStructSize      =*/ sizeof(FINDREPLACE),		// size of this struct 0x20
   /*hwndOwner        =*/ NULL,						// handle to owner's window (set before call)
   /*hInstance        =*/ NULL,						// instance handle of.EXE that
													//   contains cust. dlg. template
   /*Flags            =*/ FR_ENABLEHOOK | FR_DOWN,	// one or more of the FR_??
   /*lpstrFindWhat    =*/ szFindText,				// ptr. to search string
   /*lpstrReplaceWith =*/ NULL,						// ptr. to replace string
   /*wFindWhatLen     =*/ sizeof(szFindText),		// size of find buffer
   /*wReplaceWithLen  =*/ 0,						// size of replace buffer
   /*lCustData        =*/ 0,						// data passed to hook fn.
   /*lpfnHook         =*/ FindReplaceHook,			// ptr. to hook fn. or NULL
   /*lpTemplateName   =*/ NULL						// custom template name
};


//===========================================================================
HWND EditorFindDlg( HWND hwnd )
//===========================================================================
{
    fr.hwndOwner = hwnd;
    return FindText( &fr );
}

//===========================================================================
HWND EditorReplaceDlg (HWND hwnd)
//===========================================================================
{
    static FINDREPLACE fr;       // must be static for modeless dialog

    fr.lStructSize      = sizeof(FINDREPLACE);
    fr.hwndOwner        = hwnd;
    fr.hInstance        = NULL;
    fr.Flags            = FR_ENABLEHOOK | FR_DOWN;
    fr.lpstrFindWhat    = szFindText;
    fr.lpstrReplaceWith = szReplText;
    fr.wFindWhatLen     = sizeof(szFindText);
    fr.wReplaceWithLen  = sizeof(szReplText);
    fr.lCustData        = 0;
    fr.lpfnHook         = FindReplaceHook;
    fr.lpTemplateName   = NULL;

    return ReplaceText( &fr );
}

//===========================================================================
BOOL HandleFindText( HWND hwndEdit, int *piSearchOffset, LPFINDREPLACE pfr )
//===========================================================================
{
	FINDTEXTEX ft;

    SendMessage( hwndEdit, EM_EXGETSEL, 0, (LPARAM)&ft.chrg );
	if ( pfr->Flags & FR_DOWN ) {
		ft.chrg.cpMin = ft.chrg.cpMax;	// Search from the end of the current selection
		ft.chrg.cpMax = -1;				// to the end of the script
	}
	else {
		ft.chrg.cpMax = ft.chrg.cpMin;	// Search from the beginning of the current selection
		ft.chrg.cpMax = 0;				// to the beginning of the script
	}
	ft.lpstrText = pfr->lpstrFindWhat;

	long lpos = SendMessage( hwndEdit, EM_FINDTEXTEX, pfr->Flags, (LPARAM)&ft );
	if ( lpos > -1 )
		SendMessage( hwndEdit, EM_EXSETSEL, 0, (LPARAM)&ft.chrgText );

    return lpos;
}

//===========================================================================
BOOL EditorNextText( HWND hwndEdit, int *piSearchOffset )
//===========================================================================
{
    return HandleFindText( hwndEdit, piSearchOffset, &fr );
}

//===========================================================================
BOOL HandleReplaceText( HWND hwndEdit, int *piSearchOffset, LPFINDREPLACE pfr )
//===========================================================================
{
	// Check to see if text to be replaced is already selected
	FINDTEXTEX ft;
    SendMessage( hwndEdit, EM_EXGETSEL, 0, (LPARAM)&ft.chrg );
	ft.lpstrText = pfr->lpstrFindWhat;
	long lpos = SendMessage( hwndEdit, EM_FINDTEXTEX, pfr->Flags, (LPARAM)&ft );

	if ( lpos == -1 )	// Find the text
	    return HandleFindText(hwndEdit, piSearchOffset, pfr);

    // Replace it
    SendMessage( hwndEdit, EM_REPLACESEL, TRUE, (LPARAM) pfr->lpstrReplaceWith );

	// Find the next occurance
	return HandleFindText(hwndEdit, piSearchOffset, pfr);
}

//===========================================================================
extern HWND hDlgModeless;
//===========================================================================
STDAPI enaEditMenuFind( HWND hwndEdit, HWND hwnd )
{
	hDlgModeless = EditorFindDlg( hwnd );
	return S_OK;
}

//===========================================================================
static int       iOffset;

//===========================================================================
STDAPI enaEditMenuFindNext( HWND hwndEdit )
//===========================================================================
{
	SendMessage( hwndEdit, EM_GETSEL, NULL, (LPARAM)&iOffset );
	EditorNextText( hwndEdit, &iOffset );
	return S_OK;
}

//===========================================================================
STDAPI enaEditMenuReplace( HWND hwndEdit )
//===========================================================================
{
	hDlgModeless = EditorReplaceDlg( hwndEdit );
	return S_OK;
}

//===========================================================================
STDAPI HandleFindReplace( HWND hwndEdit, LPFINDREPLACE pfr )
//===========================================================================
{
	if ( pfr->Flags & FR_DIALOGTERM )
		hDlgModeless = NULL;
		
	if ( pfr->Flags & FR_FINDNEXT )
		if ( HandleFindText(hwndEdit, &iOffset, pfr) == -1 )
			DisplayMessage( hDlgModeless, _T("Text not found!") );
			
	if ( pfr->Flags & FR_REPLACE ||	pfr->Flags & FR_REPLACEALL )
		if ( HandleReplaceText(hwndEdit, &iOffset, pfr) == -1 )
				DisplayMessage( hDlgModeless, _T("Text not found!") );
				
	if ( pfr->Flags & FR_REPLACEALL )
		while ( HandleReplaceText (hwndEdit, &iOffset, pfr) != -1 )
			;
				
	return 0;
}
