// EditView.h : interface of the CMyEditView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_EDITVIEW_H__5375682D_5FF2_4D94_AD84_FF8DA35BF013__INCLUDED_)
#define AFX_EDITVIEW_H__5375682D_5FF2_4D94_AD84_FF8DA35BF013__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class CMyEditView : public CWindowImpl<CMyEditView, CRichEditCtrl>
{
public:
	DECLARE_WND_SUPERCLASS(NULL, pcszEnableEditClassName)

	BOOL PreTranslateMessage(MSG* pMsg)
	{
		pMsg;
		return FALSE;
	}

	enum
	{
		cchTAB = 4,					// EnableEdit42 tabsize is 4
		nSearchStringLen = 128,
		nMaxBufferLen = 0x100000	// 1 megabyte
	};

	CFont m_font;
	TCHAR m_strFilePath[MAX_PATH];
	TCHAR m_strFileName[MAX_PATH];
	CFindReplaceDialog* m_pFindDlg;
	TCHAR m_strFind[nSearchStringLen];
	BOOL m_bMatchCase;
	BOOL m_bWholeWord;
	BOOL m_bFindOnly;

	HANDLE m_hCtx;

	CMyEditView() : 
		m_pFindDlg(NULL), m_bMatchCase(FALSE), m_bWholeWord(FALSE),
		m_bFindOnly(TRUE), m_hCtx(0)
	{
		m_font = (HFONT)::GetStockObject(ANSI_FIXED_FONT);
		m_strFilePath[0] = 0;
		m_strFileName[0] = 0;
		m_strFind[0] = 0;
	}

	BEGIN_MSG_MAP(CMyEditView)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(CFindReplaceDialog::GetFindReplaceMsg(), OnFindReplaceCmd)
	ALT_MSG_MAP(1)
		COMMAND_ID_HANDLER(ID_EDIT_UNDO, OnEditUndo)
		COMMAND_ID_HANDLER(ID_EDIT_CUT, OnEditCut)
		COMMAND_ID_HANDLER(ID_EDIT_COPY, OnEditCopy)
		COMMAND_ID_HANDLER(ID_EDIT_PASTE, OnEditPaste)
		COMMAND_ID_HANDLER(ID_EDIT_CLEAR, OnEditClear)
		COMMAND_ID_HANDLER(ID_EDIT_SELECT_ALL, OnEditSelectAll)
		COMMAND_ID_HANDLER(ID_EDIT_FIND, OnEditFind)
		COMMAND_ID_HANDLER(ID_EDIT_REPEAT, OnEditFindNext)
		COMMAND_ID_HANDLER(ID_EDIT_REPLACE, OnEditReplace)
		COMMAND_ID_HANDLER(ID_FORMAT_FONT, OnViewFormatFont)

		// Messages from Debug menu
		COMMAND_ID_HANDLER(ID_DEBUG_STEPINTO, OnDebugStepInto)
		COMMAND_ID_HANDLER(ID_DEBUG_STEPOVER, OnDebugStepOver)
		COMMAND_ID_HANDLER(ID_DEBUG_QUICKWATCH, OnDebugQuickWatch)
		COMMAND_ID_HANDLER(ID_DEBUG_BREAKPOINT, OnDebugBreakpoint)
		COMMAND_ID_HANDLER(ID_DEBUG_CLEARBRKPNTS, OnDebugClearBreakpoints)
		
		// Messages from Run menu
		COMMAND_ID_HANDLER(ID_RUN_START, OnRunStart)
		COMMAND_ID_HANDLER(ID_RUN_END, OnRunEnd)
	END_MSG_MAP()

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		LRESULT lRet = DefWindowProc(uMsg, wParam, lParam);

		SetEventMask(ENM_SELCHANGE);
		LimitText(nMaxBufferLen);
		SetFont(m_font);

		return lRet;
	}

	LRESULT OnEditUndo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		Undo();
		return 0;
	}

	LRESULT OnEditCut(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		Cut();
		return 0;
	}

	LRESULT OnEditCopy(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		Copy();
		return 0;
	}

	LRESULT OnEditPaste(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		PasteSpecial(CF_TEXT);
		return 0;
	}

	LRESULT OnEditClear(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		Clear();
		return 0;
	}

	LRESULT OnEditSelectAll(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		SetSel(0, -1);
		return 0;
	}

	LRESULT OnEditFind(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		if(m_pFindDlg != NULL)
		{
			if(m_bFindOnly)
			{
				_ASSERTE(::IsWindow(m_pFindDlg->m_hWnd));
				m_pFindDlg->SetFocus();
				return 1;
			}
			else
				m_pFindDlg->DestroyWindow();
		}

		m_pFindDlg = new CFindReplaceDialog;

		if(m_pFindDlg == NULL)
		{
			_ASSERTE(FALSE);
			::MessageBeep((UINT)-1);
			return 1;
		}

		m_bFindOnly = TRUE;
		DWORD dwFlags = 0;	//FR_HIDEUPDOWN;
		if(m_bMatchCase)
			dwFlags |= FR_MATCHCASE;
		if(m_bWholeWord)
			dwFlags |= FR_WHOLEWORD;

		if(!m_pFindDlg->Create(TRUE, m_strFind, NULL, dwFlags, m_hWnd))
		{
			delete m_pFindDlg;
			m_pFindDlg = NULL;
			::MessageBeep((UINT)-1);
			return 1;
		}

		m_pFindDlg->ShowWindow(SW_NORMAL);
		return 0;
	}

	LRESULT OnEditFindNext(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		if(m_strFind[0] == 0)
			return OnEditFind(wNotifyCode, wID, hWndCtl, bHandled);

		BOOL bRet = DoFindText();
		if(!bRet)
			::MessageBeep((UINT)-1);

		return 0;
	}

	LRESULT OnFindReplaceCmd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CFindReplaceDialog* pDlg = CFindReplaceDialog::GetNotifier(lParam);
		if(pDlg == NULL)
		{
			::MessageBeep((UINT)-1);
			return 1;
		}
		_ASSERTE(pDlg == m_pFindDlg);

		if(pDlg->FindNext())
		{
			lstrcpyn(m_strFind, pDlg->m_fr.lpstrFindWhat, nSearchStringLen);
			m_bMatchCase = pDlg->MatchCase();
			m_bWholeWord = pDlg->MatchWholeWord();

			BOOL bRet = DoFindText();
			if(!bRet)
				::MessageBeep((UINT)-1);
		}
		else if(pDlg->ReplaceCurrent())
		{
			lstrcpyn(m_strFind, pDlg->m_fr.lpstrFindWhat, nSearchStringLen);
			m_bMatchCase = pDlg->MatchCase();
			m_bWholeWord = pDlg->MatchWholeWord();

			CHARRANGE chrg;
			GetSel(chrg);

			if(chrg.cpMin != chrg.cpMax)
			{
				USES_CONVERSION;
				LPSTR lpstrTextA = (LPSTR)_alloca(chrg.cpMax - chrg.cpMin + 2);
				GetSelText(lpstrTextA);
				LPTSTR lpstrText = A2T(lpstrTextA);
				int nRet;
				if(m_bMatchCase)
					nRet = lstrcmp(lpstrText, m_strFind);
				else
					nRet = lstrcmpi(lpstrText, m_strFind);
				if(nRet == 0)
					ReplaceSel(pDlg->GetReplaceString(), TRUE);
			}

			BOOL bRet = DoFindText();
			if(!bRet)
				::MessageBeep((UINT)-1);

		}
		else if(pDlg->ReplaceAll())
		{
			lstrcpyn(m_strFind, pDlg->m_fr.lpstrFindWhat, nSearchStringLen);
			m_bMatchCase = pDlg->MatchCase();
			m_bWholeWord = pDlg->MatchWholeWord();

			HCURSOR hOldCursor = NULL;
			SetRedraw(FALSE);
			BOOL bRet = DoFindText(FALSE);
			if(!bRet)
				::MessageBeep((UINT)-1);
			else
			{
				hOldCursor = ::SetCursor(::LoadCursor(NULL, IDC_WAIT));
				do
				{
					ReplaceSel(pDlg->GetReplaceString(), TRUE);
				}
				while(DoFindText(FALSE));
			}
			SetRedraw(TRUE);
			Invalidate();
			UpdateWindow();
			if(hOldCursor != NULL)
				::SetCursor(hOldCursor);
		}
		else if(pDlg->IsTerminating())
			m_pFindDlg = NULL;

		return 0;
	}

	BOOL DoFindText(BOOL bNext = TRUE)
	{
		DWORD dwFlags = FR_DOWN;
		if(m_bMatchCase)
			dwFlags |= FR_MATCHCASE;
		if(m_bWholeWord)
			dwFlags |= FR_WHOLEWORD;

		CHARRANGE chrg;
		GetSel(chrg);

		FINDTEXTEX ft;
		ft.chrg.cpMin = bNext ? chrg.cpMax : chrg.cpMin;
		ft.chrg.cpMax = -1;
		ft.lpstrText = m_strFind;

		if(FindText(dwFlags, ft) == -1)
			return FALSE;

		SetSel(ft.chrgText);

		return TRUE;
	}

	LRESULT OnEditReplace(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		if(m_pFindDlg != NULL)
		{
			if(!m_bFindOnly)
			{
				_ASSERTE(::IsWindow(m_pFindDlg->m_hWnd));
				m_pFindDlg->SetFocus();
				return 1;
			}
			else
				m_pFindDlg->DestroyWindow();
		}

		m_pFindDlg = new CFindReplaceDialog;

		if(m_pFindDlg == NULL)
		{
			_ASSERTE(FALSE);
			::MessageBeep((UINT)-1);
			return 1;
		}

		m_bFindOnly = FALSE;
		DWORD dwFlags = FR_HIDEUPDOWN;
		if(m_bMatchCase)
			dwFlags |= FR_MATCHCASE;
		if(m_bWholeWord)
			dwFlags |= FR_WHOLEWORD;

		if(!m_pFindDlg->Create(FALSE, m_strFind, NULL, dwFlags, m_hWnd))
		{
			delete m_pFindDlg;
			m_pFindDlg = NULL;
			::MessageBeep((UINT)-1);
			return 1;
		}

		m_pFindDlg->ShowWindow(SW_NORMAL);
		return 0;
	}

	LRESULT OnViewFormatFont(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		CFontDialog dlg;
		m_font.GetLogFont(&dlg.m_lf);
		dlg.m_cf.Flags |= CF_INITTOLOGFONTSTRUCT;
		if (dlg.DoModal() == IDOK)
		{
			m_font.DeleteObject();
			m_font.CreateFontIndirect(&dlg.m_lf);
			SetFont(m_font);
		}
		return 0;
	}

	HRESULT CompileScript()
	{
		HRESULT hr = enaCreateContext( _T("Enable Sample Editor"), 0, m_hWnd, &m_hCtx );
		SendMessage( SEM_SETCONTEXT, (WPARAM)m_hCtx );

		if ( SUCCEEDED(hr) )
			hr = enaAppendEditWindow( m_hCtx, m_hWnd );
		if ( SUCCEEDED(hr) )
			hr = enaCompile( m_hCtx );

		return hr;
	}

	void EndScriptRun()
	{
		enaFreeContext( m_hCtx );
		m_hCtx = 0;
		SendMessage( SEM_SETCONTEXT, 0 );
	}

		// Messages from Debug menu
	LRESULT OnDebugStepInto(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		HRESULT hr;

		if ( m_hCtx == 0 )
			hr = CompileScript();

		if ( m_hCtx )
			hr = enaDebugStepInto( m_hCtx );

		if ( hr != ENS_HIT_BREAKPOINT )
			EndScriptRun();

		return hr;
	}

	LRESULT OnDebugStepOver(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		HRESULT hr;

		if ( m_hCtx == 0 )
			hr = CompileScript();

		if ( m_hCtx )
			hr = enaDebugStep( m_hCtx );

		if ( hr != ENS_HIT_BREAKPOINT )
				EndScriptRun();

		return hr;
	}

	LRESULT OnDebugQuickWatch(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		TCHAR buf[80];
		memset( buf, 0, 80 );
		TEXTRANGE tr;
		SendMessage( EM_EXGETSEL, 0, (LPARAM)&tr.chrg );
		if ( tr.chrg.cpMax == tr.chrg.cpMin ) {
			//WB_RIGHTBREAK Finds the next end-of-word delimiter after the specified position.
			tr.chrg.cpMax = SendMessage( EM_FINDWORDBREAK, WB_RIGHTBREAK, tr.chrg.cpMax );
			//WB_MOVEWORDLEFT Finds the next character that begins a word before the specified position.
			tr.chrg.cpMin = SendMessage( EM_FINDWORDBREAK, WB_MOVEWORDLEFT, tr.chrg.cpMax );
		}
		LPSTR lpsz = (char*)_alloca((tr.chrg.cpMax - tr.chrg.cpMin + 1)*sizeof(TCHAR));
		tr.lpstrText = lpsz; 
		lpsz[0] = NULL;
		SendMessage( EM_GETTEXTRANGE, 0, (LPARAM)&tr );
		enaGetVarValueString( m_hCtx, lpsz, buf, sizeof(buf) );
		MessageBox( buf, _T("Enable Quick Watch") );
		return 0;
	}

	LRESULT OnDebugBreakpoint(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		return SendMessage( SEM_BREAKPOINT );
	}

	LRESULT OnDebugClearBreakpoints(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		return SendMessage( SEM_CLEARBRKPNTS,	0, 0 );
	}
		
		// Messages from Run menu
	LRESULT OnRunStart(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		HRESULT hr = S_OK;

		if ( m_hCtx )
			 hr = enaDebugGo( m_hCtx  );
		else {
			hr = CompileScript();
			if ( SUCCEEDED(hr) )
				hr = enaExecute( m_hCtx );
		}

		if ( hr != ENS_HIT_BREAKPOINT )
			EndScriptRun();

		return 0;
	}

	LRESULT OnRunEnd(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		EndScriptRun();
		return 0;
	}

	BOOL LoadFile(LPCTSTR lpstrFilePath)
	{
		_ASSERTE(lpstrFilePath != NULL);

		HANDLE hFile = ::CreateFile(lpstrFilePath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
		if(hFile == NULL)
			return FALSE;

		EDITSTREAM es;
		es.dwCookie = (DWORD)hFile;
		es.dwError = 0;
		es.pfnCallback = StreamReadCallback;
		StreamIn(SF_TEXT, es);

		::CloseHandle(hFile);

		return !(BOOL)es.dwError;
	}

	static DWORD CALLBACK StreamReadCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb)
	{
		_ASSERTE(dwCookie != 0);
		_ASSERTE(pcb != NULL);

		return !::ReadFile((HANDLE)dwCookie, pbBuff, cb, (LPDWORD)pcb, NULL);
	}

	BOOL SaveFile(LPTSTR lpstrFilePath)
	{
		_ASSERTE(lpstrFilePath != NULL);

		HANDLE hFile = ::CreateFile(lpstrFilePath, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
		if(hFile == NULL)
			return FALSE;

		EDITSTREAM es;
		es.dwCookie = (DWORD)hFile;
		es.dwError = 0;
		es.pfnCallback = StreamWriteCallback;
		StreamOut(SF_TEXT, es);

		::CloseHandle(hFile);

		return !(BOOL)es.dwError;
	}

	static DWORD CALLBACK StreamWriteCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb)
	{
		_ASSERTE(dwCookie != 0);
		_ASSERTE(pcb != NULL);

		return !::WriteFile((HANDLE)dwCookie, pbBuff, cb, (LPDWORD)pcb, NULL);
	}

	void Init(LPCTSTR lpstrFilePath, LPCTSTR lpstrFileName)
	{
		lstrcpy(m_strFilePath, lpstrFilePath);
		lstrcpy(m_strFileName, lpstrFileName);
		SetModify(FALSE);
	}

	BOOL QueryClose()
	{
		EndScriptRun();

		if(!GetModify())
			return TRUE;

		CWindow wndMain(GetParent());
		TCHAR szBuff[MAX_PATH + 30];
		wsprintf(szBuff, _T("Save changes to %s ?"), m_strFileName);
		int nRet = wndMain.MessageBox(szBuff, _T("Edit"), MB_YESNOCANCEL | MB_ICONEXCLAMATION);

		if(nRet == IDCANCEL)
			return FALSE;
		else if(nRet == IDYES)
		{
			if(!DoFileSaveAs())
				return FALSE;
		}

		return TRUE;
	}

	BOOL DoFileSaveAs()
	{
		BOOL bRet = FALSE;

		CFileDialog dlg(FALSE, NULL, m_strFilePath, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, lpcstrFilter);
		int nRet = dlg.DoModal();

		if(nRet == IDOK)
		{
			ATLTRACE(_T("File path: %s\n"), dlg.m_ofn.lpstrFile);
			bRet = SaveFile(dlg.m_ofn.lpstrFile);
			if(bRet)
				Init(dlg.m_ofn.lpstrFile, dlg.m_ofn.lpstrFileTitle);
			else
				MessageBox(_T("Error writing file!\n"));
		}

		return bRet;
	}
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_EDITVIEW_H__5375682D_5FF2_4D94_AD84_FF8DA35BF013__INCLUDED_)
