// ScriptView.cpp : implementation of the CScriptEditView class
//

#include "stdafx.h"
#include "ScriptView.h"
#include "..\..\..\Enable.h"
#include "resource.h"

#pragma comment(lib, "..\\..\\..\\Debug\\Enable40.lib")

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScriptEditView

IMPLEMENT_DYNCREATE(CScriptEditView, CEditView)

BEGIN_MESSAGE_MAP(CScriptEditView, CEditView)
	//{{AFX_MSG_MAP(CScriptEditView)
 	ON_UPDATE_COMMAND_UI(ID_RUN_START, OnUpdateRunStart)
 	ON_UPDATE_COMMAND_UI(ID_RUN_END, OnUpdateRunEnd)
 	ON_COMMAND(ID_DEBUG_STEP_INTO, OnDebugStepInto)
 	ON_COMMAND(ID_DEBUG_STEP_OVER, OnDebugStepOver)
 	ON_COMMAND(ID_DEBUG_BREAKPOINT, OnDebugToggleBreakpoint)
 	ON_COMMAND(ID_DEBUG_CLEAR_BREAKPOINTS, OnDebugClearBreakpoints)
 	ON_COMMAND(ID_RUN_START, OnRunStart)
 	ON_COMMAND(ID_RUN_END, OnRunEnd)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScriptEditView construction/destruction

CScriptEditView::CScriptEditView()
{
	m_hCtx = 0;
	m_pDefDispatch = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// CScriptEditView overrides

BOOL CScriptEditView::PreCreateWindow(CREATESTRUCT& cs)
{
	enaInitEnableEdit( 0 );
	m_strClass = _T("EnableEdit42");

	BOOL bPreCreated = CEditView::PreCreateWindow(cs);
	cs.style |= WS_CLIPCHILDREN | ES_MULTILINE | WS_HSCROLL | WS_VSCROLL |
			ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_DISABLENOSCROLL |
			ES_NOHIDESEL | ES_SAVESEL | ES_SELECTIONBAR;

	return bPreCreated;
}

BOOL CScriptEditView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	BOOL bCreate = CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);

	SendMessage( EM_LIMITTEXT, 0x100000);	// Set max file size to one megabyte
	SendMessage( WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT) );

	// We want selection change notification for Ln, Col,  
	SendMessage( EM_SETEVENTMASK, 0, ENM_CHANGE | ENM_SELCHANGE );

	// Load type library for script tips and syntax coloring
	ITypeLib *pTypeLib;
	HRESULT hr = LoadTypeLib( L"SDIEdit.tlb", &pTypeLib );
	if ( SUCCEEDED(hr) ) {
		SendMessage( SEM_ADDTYPELIBREF, (WPARAM)pTypeLib );
		pTypeLib->Release();
	}

	DWORD dwColorTable [] = {
		-9999997,		// Auto   - normal script text color
		0xff0000,		// Blue   - keywords
		0x008000,		// Green  - comments
		0x008080,		// Olive  - built in functions
		-9999997,		// Auto   - identifiers
		0x800080,		// Purple - numbers
		0x0000ff,		// Red    - strings
		0x000080		// DkRed  - constants
	};
	SendMessage( SEM_SETCOLORTABLE, sizeof(dwColorTable), (LPARAM)dwColorTable );

	SendMessage( SEM_SETEDITOPTIONS, 1 );	// Color while editing

	return bCreate;
}

/////////////////////////////////////////////////////////////////////////////
// CScriptEditView implementation

//=============================================================================================
BOOL CScriptEditView::OpenFile(LPCTSTR lpszPathName)
//=============================================================================================
{
	COleVariant var( lpszPathName );
	HRESULT hr = SendMessage( SEM_OPEN, (WPARAM)&var, 0 );
	return SUCCEEDED( hr );
}

//=============================================================================================
BOOL CScriptEditView::SaveFile(LPCTSTR lpszPathName)
//=============================================================================================
{
	COleVariant var( lpszPathName );
	HRESULT hr = SendMessage( SEM_SAVE, (WPARAM)&var, 0 );
	return SUCCEEDED( hr );
}

const TCHAR *pAppTitle = _T("Cypress Enable SDIEdit");

//=============================================================================================
HRESULT CScriptEditView::DisplayResultCodeMessage(HRESULT hr)
//=============================================================================================
{
	TCHAR *pMsgBuf = 0;
	DWORD dwCount = FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, hr, 
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (TCHAR *)&pMsgBuf, 0, NULL );
	MessageBox( dwCount ? pMsgBuf : _T("Error running script"), pAppTitle, MB_ICONWARNING );
	LocalFree( pMsgBuf );

	return hr;
}

//=============================================================================================
HRESULT CScriptEditView::PrepareContext(HANDLE &hCtx)
//=============================================================================================
{
	// Create the context
    HRESULT hr = enaCreateContext( pAppTitle, 0, m_hWnd, &hCtx );

	// Set options, add references, register callbacks and set default object
	if ( SUCCEEDED(hr) ) {
		enaSetOption( hCtx, ENOPT_CALL_APP_WITH_CTX, TRUE );	// Implicit context as first parameter
		enaAddTypeLibFileRefW( hCtx, L"SDIEdit.exe", 0 );		// Type library embedded in exe
		enaSetDefaultDispatch( hCtx, 0, m_pDefDispatch );
	}

	return hr;
}

//=============================================================================================
HRESULT CScriptEditView::CompileEditWindowScript()
//=============================================================================================
{
    HRESULT hr = PrepareContext( m_hCtx );

	if ( SUCCEEDED(hr) )
		hr = enaAppendEditWindow( m_hCtx, m_hWnd );

	if ( SUCCEEDED(hr) )
		hr = enaCompile( m_hCtx );		// Compile errors are reported by Enable
	else
		DisplayResultCodeMessage( hr );	// Report any failed HRESULTs

	if ( SUCCEEDED(hr) ) {

		// Add code here to set global variables, etc.
	}
	else {
		enaFreeContext( m_hCtx );
		m_hCtx = 0;
	}

	return hr;
}

//=============================================================================================
void CScriptEditView::FreeContext()
//=============================================================================================
{
	if ( m_hCtx ) {
		// Add code here to get global variables, etc.
		enaFreeContext( m_hCtx );
		m_hCtx = 0;
	}
}

//=============================================================================================
HRESULT CScriptEditView::RunScript(LPCTSTR lpszScriptText) 
//=============================================================================================
{
	HANDLE	hCtx;
    HRESULT hr = PrepareContext( hCtx );

	if ( SUCCEEDED(hr) )
		hr = enaAppendText( hCtx, lpszScriptText );

	if ( SUCCEEDED(hr) )
		hr = enaCompile( hCtx );			// Compile errors are reported by Enable
	else
		DisplayResultCodeMessage( hr );		// Report any failed HRESULTs

	if ( SUCCEEDED(hr) )
		hr = enaExecute( hCtx );			// Runtime errors are reported by Enable

	enaFreeContext( hCtx );

	return hr;
}

/////////////////////////////////////////////////////////////////////////////
// CScriptEditView message handlers

void CScriptEditView::OnDebugStepInto()
{
	if ( m_hCtx == NULL )
		CompileEditWindowScript();

	if ( m_hCtx && enaDebugStepInto(m_hCtx) != ENS_HIT_BREAKPOINT )
		FreeContext();
}

void CScriptEditView::OnDebugStepOver()
{
	if ( m_hCtx == NULL )
		CompileEditWindowScript();

	if ( m_hCtx && enaDebugStep(m_hCtx) != ENS_HIT_BREAKPOINT )
		FreeContext();
}

void CScriptEditView::OnDebugToggleBreakpoint()
{
	SendMessage( SEM_BREAKPOINT );
}

void CScriptEditView::OnDebugClearBreakpoints()
{
	SendMessage( SEM_CLEARBRKPNTS );
}

void CScriptEditView::OnUpdateRunStart(CCmdUI* pCmdUI)
{
	pCmdUI->SetText( m_hCtx ? _T("&Continue\tF5") : _T("&Start\tF5") );
}

void CScriptEditView::OnRunStart()
{
	if ( m_hCtx == NULL )
		CompileEditWindowScript();

	if ( m_hCtx && enaExecute(m_hCtx) != ENS_HIT_BREAKPOINT )
		FreeContext();
}

void CScriptEditView::OnUpdateRunEnd(CCmdUI* pCmdUI)
{
	pCmdUI->Enable( m_hCtx != NULL );
}

void CScriptEditView::OnRunEnd()
{
	FreeContext();
}

void CScriptEditView::OnDestroy()
{
	FreeContext();
	CEditView::OnDestroy();
}
/////////////////////////////////////////////////////////////////////////////
